#ifndef CENGINE_H
#define CENGINE_H

#include "libboxmanage_global.h"

#include <QList>

#define BOX_EXPORT              0
#define BOX_IMPORT              1
#define BOX_SUCCESS             0
#define BOX_FAILURE             1
#define BOX_FAILURE_NOBOX       2
namespace Box {

typedef struct {
    QString strName;
    QString strPath;
    QString strOwner;
    QString strTime;
    int nEncrypted;     //1:加密 0：未加密
    int nMounted;       //1:未上锁 0：上锁
}SBoxItem;

class LIBBOXMANAGE_EXPORT CEngine
{
public:
    static CEngine *getInstance()
    {
        static CEngine *_instance = nullptr;

        if(!_instance) {
            _instance = new CEngine();
        }

        return _instance;
    }

    virtual ~CEngine();

    bool get_enable();

    int get_boxList(QList<SBoxItem> &boxList);
    int get_boxInfoByName(const QString &strName, SBoxItem & boxInfo);

    int get_boxTSByName(const QString &strName);

    int create_normalBox(const QString &strName);
    int create_encryptBox(const QString &strName, const QString &strPasswd);
    int create_transparentBox(const QString &strName);

    int delete_box(const QString &strName);

    int change_boxName(const QString &strOldName, const QString &strNewName);
    int change_boxPasswd(const QString &strName, const QString &strOldPasswd, const QString &strNewPasswd);

    bool verify_boxPassphrase(const QString &strName, const QString &strPasswd);
    bool verify_boxPassphraseByBoxSM(const QString &strBoxname, const QString &newPasswd);

    bool verify_boxPassphrase_v2(const QString &strName, const QString &strPasswd, QString &error);
    bool verify_boxPassphraseByBoxSM_v2(const QString &strBoxname, const QString &newPasswd, QString &error);

    int mount_ecnryptedBox(const QString &strName, const QString strPasswd);
    int mount_ecnryptedBoxByBoxSM(const QString &strName, const QString strPasswd);

    int mount_ecnryptedBoxByGlobalKey(const QString &strName, const QString strKey);
    int mount_ecnryptedBoxByGlobalKeyByBoxSM(const QString &strName, const QString strKey);

    int umount_encryptedBox(const QString &strName);
    int umount_encryptedBoxByBoxSM(const QString &strName);

    bool is_boxExist(const QString &box_name);
    int change_boxPasswdByGlobalKey(const QString &strBoxname, const QString &pswTxt, const QString &newPasswd);

    int create_GlobalKey(QString &strKey);
    int create_GlobalKeyByBoxSM(QString &strKey, QString keyFile);

    int remove_GlobalKey();
    int remove_GlobalKeyByBoxSM();

    bool verify_GlobalKey(const QString &pswTxt);
    bool verify_GlobalKeyByBoxSM(const QString &pswTxt);
    bool box_GlobalKeyExist();
    bool box_GlobalKeyExistByBoxSM();

    int create_normalBoxByBoxSM(const QString &strName);
    int create_encryptBoxByBoxSM(const QString &strName, const QString &strPasswd);
    int create_transparentBoxByBoxSM(const QString &strName);

    int change_boxNameByBoxSM(const QString &strOldName, const QString &strNewName);
    int change_boxNameByBoxSM(const QString &strOldName, const QString &strNewName, const QString &password);
    int change_boxPasswdByBoxSM(const QString &strName, const QString &strOldPasswd, const QString &strNewPasswd);
    int change_boxPasswdUseBoxSMByGlobalKey(const QString &strBoxname, const QString &pswTxt, const QString &newPasswd);

    int delete_boxByBoxSM(const QString &strName);
    int delete_boxByBoxSM(const QString &strName, const QString &password);

    int export_boxByBoxSM(const QString &strName,const QString &strPath, const QString &password);
    int import_boxByBoxSM(const QString &strPath, const QString &password, const QString &strName, const QString &oldpassword);


    int get_boxListByBoxSM(QList<SBoxItem> &boxList);
    int get_boxInfoByNameByBoxSM(const QString &strName, SBoxItem &boxInfo);

    int create_builtinBox(QString tsBox, QString encBox);
    int create_builtinBoxByBoxSM(QString tsBox, QString encBox);
    bool verify_builtinBox(QString boxName);
    bool verify_builtinBoxByBoxSM(QString boxName);
    int remove_builtinFlag(QString boxName);
    int remove_builtinFlagByBoxSM(QString boxName);

    int check_box_import_encrypted(QString strPath);

    bool check_fsresetkeyUsable(QString boxName);
    bool check_fsresetkeyUsableByBoxSM(QString boxName);

    bool check_compatibilityUpgradeStatus();
    bool check_compatibilityUpgradeStatusByBoxSM();

    int begin_compatibilityUupgrade();
    int begin_compatibilityUupgradeByBoxSM();

    int init_box();

    QString get_currentUserName();

private:
    CEngine();

private:
    bool m_enable;
};

}

#endif // CENGINE_H
