#ifndef TRACKERMONITOR_H
#define TRACKERMONITOR_H

#include "libboxmanage_global.h"
#include "kysdk/diagnosetest/libkydatacollect.h"

#include <QObject>
#include <QDebug>
#include <unordered_map>

class LIBBOXMANAGE_EXPORT TrackerMonitor :  public QObject {
    Q_OBJECT
  public:
    static TrackerMonitor &getInstance() {
        static TrackerMonitor *instance = new TrackerMonitor();
        return *instance;
    }

    static void uploadProgramStartUp();
    static void uploadProgramExit();
    static void uploadSearchContent(char *eventCode, char *pageName, char *content);

    // custom event
    static void uploadEvent(char *eventCode, char *pageName, KEventType eventType);
    static void uploadEventCustom(char *eventCode, char *pageName, KEventType eventType, const std::unordered_map<char *, char *> &propertyMap);

    /**
     * @brief 将埋点上传的流程注入到相应的信号中
     * TODO: 需要定制信号与事件，不能仅考虑点击
     */
    template <typename T, typename... Args>
    void injectSignalClick(T *t, void (T::*clickSignal)(Args...), const QString &eventObject, const QString &pageName) {
        connect(t, clickSignal, this, [ = ]() {
            qDebug() << "[TrackerMonitor] injectSignalClick clickObject = " << eventObject << ", pageName = " << pageName;
            std::unordered_map<char *, char *> paramsMap;
            paramsMap[const_cast<char *>("eventObject")] = eventObject.toLocal8Bit().data();
            uploadEventCustom(const_cast<char *>("box_manager_click_event"), pageName.toLocal8Bit().data(), KEVENT_CLICK, paramsMap);
        });
    }

  protected:
    TrackerMonitor() {}

  private:
    TrackerMonitor(const TrackerMonitor &) = delete;
    TrackerMonitor &operator=(const TrackerMonitor &) = delete;
};

#endif  // TRACKERMONITOR_H
